import enum

from PyQt5 import QtCore
from PyQt5.QtCore import QObject, QTimer


__all__ = ["GpioState", "GpioReader"]


_gpio_value_path_template = "/sys/class/gpio/gpio{}/value"


class GpioState(enum.IntEnum):
    LOW = 0
    HIGH = 1


class GpioReader(QObject):
    state_changed = QtCore.pyqtSignal(GpioState)

    def __init__(self, pin: int, frequency: float = 5.0, parent=None) -> None:
        super().__init__(parent=parent)

        self.__state = None

        self.__pin = pin
        self.__file_path = _gpio_value_path_template.format(self.__pin)

        self.__timer = QTimer(parent=self)
        self.__timer.timeout.connect(self.__on_timeout)
        self.__timer.setInterval(int(1000.0 / frequency))  # in milliseconds
        self.__timer.setSingleShot(False)

    @QtCore.pyqtSlot()
    def start(self) -> None:
        self.__timer.start()

    @QtCore.pyqtSlot()
    def stop(self) -> None:
        self.__timer.stop()

    @QtCore.pyqtSlot()
    def __on_timeout(self) -> None:
        with open(self.__file_path, "rb") as gpio_file:
            new_state = GpioState.HIGH if gpio_file.read(1) == b"1" else GpioState.LOW
            if new_state != self.__state:
                self.__state = new_state
                self.state_changed.emit(new_state)
