from typing import Iterable, List
import json

from element import Element


def dump_elements(file_path: str, elements: Iterable[Element]) -> None:
    # Old variant
    # joblib.dump(elements, file_path, compress=9)

    # New variant
    d = {
        "elements": [element.to_json() for element in elements]
    }
    with open(file_path, "w") as dump_file:
        json.dump(d, dump_file, separators=(",", ":"))  # Trim spaces


def load_elements(file_path) -> List[Element]:
    # Old variant
    # elements = joblib.load(file_path)

    # New variant
    with open(file_path, "r") as dump_file:
        d = json.load(dump_file)
    elements = list(map(Element.from_json, d["elements"]))

    # Was commented. TODO: Clear.
    # if len(elements) == 0:
    #     return elements
    #     for i in range(len(elements)):
    #         if not hasattr(elements[i], "side_idxes"):
    #             elements[i].side_idxes = None
    #         if len(elements[i].pins) > 0 and hasattr(elements[i].pins[0], "ivc"):
    #             for j in range(len(elements[i].pins)):
    #                 elements[i].pins[j].ivc = elements[i].pins[j].ivc
    #                 del elements[i].pins[j].ivc
    #                 if elements[i].pins[j].reference_ivc is not None:
    #                     elements[i].pins[j].reference_ivc = elements[i].pins[j].reference_ivc
    #                     del elements[i].pins[j].reference_ivc
    return elements
